// Same copyright and license as the rest of the files in this project

// GVariant : GVariant — strongly typed value datatype
// https://developer.gnome.org/glib/2.26/glib-GVariant.html

#ifndef __GVARIANTTYPE_GO_H__
#define __GVARIANTTYPE_GO_H__

const GVariantType *_G_VARIANT_TYPE_BOOLEAN = G_VARIANT_TYPE_BOOLEAN;
const GVariantType *_G_VARIANT_TYPE_BYTE = G_VARIANT_TYPE_BYTE;
const GVariantType *_G_VARIANT_TYPE_INT16 = G_VARIANT_TYPE_INT16;
const GVariantType *_G_VARIANT_TYPE_UINT16 = G_VARIANT_TYPE_UINT16;
const GVariantType *_G_VARIANT_TYPE_INT32 = G_VARIANT_TYPE_INT32;
const GVariantType *_G_VARIANT_TYPE_UINT32 = G_VARIANT_TYPE_UINT32;
const GVariantType *_G_VARIANT_TYPE_INT64 = G_VARIANT_TYPE_INT64;
const GVariantType *_G_VARIANT_TYPE_UINT64 = G_VARIANT_TYPE_UINT64;
const GVariantType *_G_VARIANT_TYPE_HANDLE = G_VARIANT_TYPE_HANDLE;
const GVariantType *_G_VARIANT_TYPE_DOUBLE = G_VARIANT_TYPE_DOUBLE;
const GVariantType *_G_VARIANT_TYPE_STRING = G_VARIANT_TYPE_STRING;
const GVariantType *_G_VARIANT_TYPE_OBJECT_PATH = G_VARIANT_TYPE_OBJECT_PATH;
const GVariantType *_G_VARIANT_TYPE_SIGNATURE = G_VARIANT_TYPE_SIGNATURE;
const GVariantType *_G_VARIANT_TYPE_VARIANT = G_VARIANT_TYPE_VARIANT;
const GVariantType *_G_VARIANT_TYPE_ANY = G_VARIANT_TYPE_ANY;
const GVariantType *_G_VARIANT_TYPE_BASIC = G_VARIANT_TYPE_BASIC;
const GVariantType *_G_VARIANT_TYPE_MAYBE = G_VARIANT_TYPE_MAYBE;
const GVariantType *_G_VARIANT_TYPE_ARRAY = G_VARIANT_TYPE_ARRAY;
const GVariantType *_G_VARIANT_TYPE_TUPLE = G_VARIANT_TYPE_TUPLE;
const GVariantType *_G_VARIANT_TYPE_UNIT = G_VARIANT_TYPE_UNIT;
const GVariantType *_G_VARIANT_TYPE_DICT_ENTRY = G_VARIANT_TYPE_DICT_ENTRY;
const GVariantType *_G_VARIANT_TYPE_DICTIONARY = G_VARIANT_TYPE_DICTIONARY;
const GVariantType *_G_VARIANT_TYPE_STRING_ARRAY = G_VARIANT_TYPE_STRING_ARRAY;
const GVariantType *_G_VARIANT_TYPE_OBJECT_PATH_ARRAY =
    G_VARIANT_TYPE_OBJECT_PATH_ARRAY;
const GVariantType *_G_VARIANT_TYPE_BYTESTRING = G_VARIANT_TYPE_BYTESTRING;
const GVariantType *_G_VARIANT_TYPE_BYTESTRING_ARRAY =
    G_VARIANT_TYPE_BYTESTRING_ARRAY;
const GVariantType *_G_VARIANT_TYPE_VARDICT = G_VARIANT_TYPE_VARDICT;

#endif
